@@ -59,6 +59,9 @@ are additonal arguments used for exec-operations,
 Multiple request object are returned e.g. if a relative check has to be
 performed in order to get the base value as well.
 
+The returned array can contain coderefs which should be executed directly and
+its return value should be used in order to perfoorm the check. 
+
 =cut
 
 sub get_requests {
@@ -66,7 +69,11 @@ sub get_requests {
     my $jmx = shift;
     my $args = shift;
 
+    # If a script is given, extract a subref and return it
+    return [ $self->_extract_script_as_subref($jmx) ] if $self->script;
+
     my $do_read = $self->attribute || $self->value;
+    my $do_exec = $self->operation;
     if ($self->alias) {
         my $alias = JMX::Jmx4Perl::Alias->by_name($self->alias);
         die "No alias '",$self->alias," known" unless $alias;
@@ -76,8 +83,10 @@ sub get_requests {
     my $request;
     if ($do_read) {
         $request = JMX::Jmx4Perl::Request->new(READ,$self->_prepare_read_args($jmx));
-    } else {
+    } elsif ($do_exec) {
         $request = JMX::Jmx4Perl::Request->new(EXEC,$self->_prepare_exec_args($jmx,@$args));
+    } else {
+        die "Neither an attribute/value, an operation or a script given";
     }
     my $method = $self->{np}->opts->{method} || $self->{config}->{method};
     if ($method) {
@@ -107,6 +116,25 @@ sub get_requests {
     return \@requests;
 }
 
+# Create a subref where all params from the outside are available as closures.
+sub _extract_script_as_subref {
+    my $self = shift;
+    my $jmx = shift;
+    my $script = $self->script || die "No script given";
+    my $full_script = <<"EOT";
+sub {
+  my \$j4p = shift;
+  return sub { 
+     $script
+  }
+}
+EOT
+    #print $full_script,"\n";
+    my $sub = eval $full_script;
+    die "Cannot eval script for check ",$self->name,": $@" if $@;
+    return &$sub($jmx);
+}
+
 =item $single_check->exract_responses($responses,$requests,$target)
 
 Extract L<JMX::Jmx4Perl::Response> objects and add the deducted results to 
@@ -135,12 +163,20 @@ sub extract_responses {
     my $request = shift @{$requests};
     #print Dumper($resp);
     my @extra_requests = ();
-    $self->_verify_response($request,$resp);
-    my $value = $self->_extract_value($request,$resp);
+    my $value;
+    my $script_mode = undef;
+    if (ref($request) eq "CODE") {
+        # It's a script, so the 'response' is already the value
+        $script_mode = 1;
+        $value = $resp;
+    } else {
+        $self->_verify_response($request,$resp);
+        $value = $self->_extract_value($request,$resp);
+    }
    
     # Delta handling
     my $delta = $self->delta;
-    if (defined($delta)) {
+    if (defined($delta) && !$script_mode) {
         $value = $self->_delta_value($request,$resp,$delta);
         unless (defined($value)) {
             push @extra_requests,$self->_switch_on_history($request,$opts->{target});
@@ -151,7 +187,7 @@ sub extract_responses {
     # Normalize value 
     my ($value_conv,$unit) = $self->_normalize_value($value);
     my $label = $self->_get_name(cleanup => 1);
-    if ($self->base) {
+    if ($self->base && !$script_mode) {
         # Calc relative value 
         my $base_value = $self->_base_value($self->base,$responses,$requests);
         my $rel_value = sprintf "%2.2f",$base_value ? (int((($value / $base_value) * 10000) + 0.5) / 100) : 0;
@@ -163,7 +199,7 @@ sub extract_responses {
                           min => 0,max => $base_value,
                           $self->unit ? (uom => $self->unit) : ());
         # Do the real check.
-        my ($code,$mode) = $self->_check_threshhold($rel_value);
+        my ($code,$mode) = $self->_check_threshold($rel_value);
         # For Multichecks, we remember the label of a currently failed check
         $self->_update_error_stats($opts->{error_stat},$code) unless $code == OK;
         my ($base_conv,$base_unit) = $self->_normalize_value($base_value);
@@ -172,12 +208,13 @@ sub extract_responses {
                                                     base_unit => $base_unit, prefix => $opts->{prefix}));            
     } else {
         # Performance data
+        $value = $self->_sanitize_value($value);
         $np->add_perfdata(label => $label,
                           critical => $self->critical, warning => $self->warning, 
                           value => $value,$self->unit ? (uom => $self->unit) : ());
         
         # Do the real check.
-        my ($code,$mode) = $self->_check_threshhold($value);
+        my ($code,$mode) = $self->_check_threshold($value);
         $self->_update_error_stats($opts->{error_stat},$code) unless $code == OK;
         $np->add_message($code,$self->_exit_message(code => $code,mode => $mode,value => $value_conv, unit => $unit,
                                                     prefix => $opts->{prefix}));                    
@@ -190,7 +227,7 @@ sub _update_error_stats {
     my $error_stat = shift || return;
     my $code = shift;
 
-    my $label = $self->{config}->{key} || $self->{config}->{name};
+    my $label = $self->{config}->{name} || $self->{config}->{key};
     if ($label) {
         my $arr = $error_stat->{$code} || [];
         push @$arr,$label;
@@ -367,6 +404,14 @@ sub _normalize_value {
     die "Unknown unit '$unit' for value $value";
 }
 
+sub _sanitize_value {
+    my ($self,$value) = @_;
+    if ($value =~ /\de/i) {
+        $value = sprintf("%f", $value);
+    }
+    return $value;
+}
+
 sub _verify_response {
     my ($self,$req,$resp) = @_;
     my $np = $self->{np};
@@ -482,7 +527,7 @@ sub _split_attr_spec {
     return @ret;
 }
 
-sub _check_threshhold {
+sub _check_threshold {
     my $self = shift;
     my $value = shift;
     my $np = $self->{np};
@@ -499,7 +544,7 @@ sub _check_threshhold {
            defined($self->critical) ? (critical => $self->critical) : (),
            defined($self->warning) ? (warning => $self->warning) : ()
           );            
-        return ($np->check_threshold(check => $value,@ths),"numeric");    
+        return (@ths ? $np->check_threshold(check => $value,@ths) : OK,"numeric");    
     } else {
         return
           ($self->_check_string_threshold($value,CRITICAL,$self->critical) ||
@@ -668,6 +713,8 @@ my $CHECK_CONFIG_KEYS = {
 
                          "value" => "value",
                          "null" => "null",
+
+                         "script" => "script"
                         };
 
 
